/*---------------------------------------------------------------------------*
 *                                   IT++			             *
 *---------------------------------------------------------------------------*
 * Copyright (c) 1995-2001 by Tony Ottosson, Thomas Eriksson, Pl Frenger,   *
 * Tobias Ringstrm, and Jonas Samuelsson.                                   *
 *                                                                           *
 * Permission to use, copy, modify, and distribute this software and its     *
 * documentation under the terms of the GNU General Public License is hereby *
 * granted. No representations are made about the suitability of this        *
 * software for any purpose. It is provided "as is" without expressed or     *
 * implied warranty. See the GNU General Public License for more details.    *
 *---------------------------------------------------------------------------*/

/*!
  \file
  \brief Some it++ specific configurations and definitions

  1.15

  2003/06/24 12:20:03
*/

#ifndef __itconfig_h
#define __itconfig_h

#include <complex>

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define ITPP_DEFAULT_EXCEPTIONS 0
#endif //DOXYGEN_SHOULD_SKIP_THIS

//-------------------------------------------------------------------
// These function (min and max) are not defined in namespace std on 
// Apple (GCC 3.1). In case they fix this in a later GCC version then 
// these lines needs to be removed.
//-------------------------------------------------------------------
#if defined(__APPLE__)
namespace std {
  template <class T>
    inline const T& min(const T& a, const T& b) {
    return b < a ? b : a;
  }
  template <class T>
    inline const T& max(const T& a, const T& b) {
    return  a < b ? b : a;
  }
}
#endif //__APPLE__

//---------------------------------------------------------------

//#if (__GNUC__ > 2) || (__GNUC__ == 2 && __GNUC_MINOR__ > 95) || (__GNUC__ == 2 && __GNUC_MINOR__ == 95 && __GNUC_PATCHLEVEL__ > 2) || (_MSC_VER)

//! Output stream operator for complex numbers
namespace std {
  template <class T> std::ostream&
    operator<<(std::ostream &os, const std::complex<T> &x)
    {
      os <<  x.real() ;
      if (x.imag() >= 0) os << '+' << x.imag();
      else os << x.imag();
      return os << 'i';
    }
}

// --------- Special Visual C++ directives -----------------------------
#if  (_MSC_VER && _MSC_VER<1300)

#pragma warning( disable : 4275 )

// Disable the warning about debug symbols being longer than 255 characters
#pragma warning( disable : 4786 )

#define TEMPLATE_FUN
#ifdef min
#  undef min
#endif
#ifdef max
#  undef max
#endif

namespace std {
  template <class T>
    inline const T& min(const T& a, const T& b) {
    return b < a ? b : a;
  }
  template <class T>
    inline const T& max(const T& a, const T& b) {
    return  a < b ? b : a;
  }
}

#elif (_MSC_VER == 1300 )

#define TEMPLATE_FUN

#else
//! Define of TEMPLATE_FUN
#define TEMPLATE_FUN <>

#endif

#endif // __itconfig_h
